import jwtDecode from "jwt-decode";
import apiClient from "../api/api-client";

let _accessToken, _refreshToken, _storeTokens;

apiClient.addAsyncRequestTransform(configureAuthorization);

async function configureAuthorization(request) {
  if (shouldRefreshAccessToken(request)) await refreshAccessToken();
  if (_accessToken) request.headers['Authorization'] = `Bearer ${_accessToken}`;
}

function shouldRefreshAccessToken(request) {
  const isNotAuthRequest = !isAuthorizationRequest(request);
  const hasRefreshToken = !!_refreshToken;
  const isMissingAccessToken = !_accessToken;

  return isNotAuthRequest && hasRefreshToken && (isMissingAccessToken || isTokenMalformedOrExpired(_accessToken));
}

function isAuthorizationRequest(request) {
  return request.url.includes('/auth');
}

function getAccessTokenPayload(accessToken) {
  try {
    return jwtDecode(accessToken);
  } catch (e) {
    return undefined
  }
}

function isTokenMalformedOrExpired(accessToken) {
  const tokenPayload = getAccessTokenPayload(accessToken);
  if (!tokenPayload) return false

  const tokenExpiration = tokenPayload.exp;
  const inThirtySeconds = (new Date() / 1000) + 30;
  return tokenExpiration <= inThirtySeconds;
}

async function refreshAccessToken() {
  const {status, data, problem} = await apiClient.post('/auth/refresh', {refreshToken: _refreshToken});
  if (status === 200) {
    const {accessToken, refreshToken} = data;
    await updateTokens(accessToken, refreshToken);
  } else if (problem === 'CLIENT_ERROR') {
    await updateTokens(undefined, undefined);
  }
}

async function updateTokens(accessToken, refreshToken) {
  setTokens(accessToken, refreshToken);
  if (_storeTokens) await _storeTokens(accessToken, refreshToken);
}

function setTokens(accessToken, refreshToken) {
  _accessToken = accessToken;
  _refreshToken = refreshToken;
}

function setStoreTokensFunction(fn) {
  _storeTokens = fn;
}

export {setTokens, setStoreTokensFunction}