import constants from './constants';

function useDensityUtils() {
  let unit;

  const setDensityUnit = (newUnit) => {
    if (![constants.SPECIFIC_GRAVITY, constants.PLATO].includes(newUnit)) {
      throw new Error(`Density unit not recognised: ${newUnit}`)
    } else {
      unit = newUnit
    }
  }

  const getDensity = (specificGravity) => {
    switch (unit) {
      case constants.SPECIFIC_GRAVITY:
        return parseFloat(specificGravity);
      case constants.PLATO:
        return convertSgToPlato(specificGravity);
      default:
        return undefined
    }
  }

  const getDensityUnit = () => {
    switch (unit) {
      case constants.SPECIFIC_GRAVITY:
        return 'SG';
      case constants.PLATO:
        return '°P';
      default:
        return undefined
    }
  }

  const getDensityString = (specificGravity, withUnit = true) => {
    let density = getDensity(specificGravity);
    if (!Number.isFinite(density)) return '--';

    switch (unit) {
      case constants.SPECIFIC_GRAVITY:
        density = density.toFixed(3);
        break;
      case constants.PLATO:
        density = density.toFixed(2);
        break;
      default:
        return '--'
    }

    if (withUnit) {
      density += ` ${getDensityUnit()}`;
    }

    return density;
  }

  function convertSgToPlato(specificGravity) {
    if (!specificGravity) return specificGravity
    const plato = -616.868 + (1111.14 * specificGravity) - (630.272 * Math.pow(specificGravity, 2)) + (135.997 * Math.pow(specificGravity, 3))
    return Math.round(plato * 1000) / 1000;
  }

  const convertPlatoToSg = (plato) => {
    if (!plato) return plato
    return 1.0 + (plato / (258.6 - ((plato / 258.2) * 227.1)));
  }

  const getDensityMin = () => {
    switch (unit) {
      case constants.SPECIFIC_GRAVITY:
        return constants.SPECIFIC_GRAVIY_MIN;
      case constants.PLATO:
        return constants.PLATO_MIN;
      default:
        return undefined
    }
  }

  const getDensityMax = () => {
    switch (unit) {
      case constants.SPECIFIC_GRAVITY:
        return constants.SPECIFIC_GRAVIY_MAX;
      case constants.PLATO:
        return constants.PLATO_MAX;
      default:
        return undefined
    }
  }

  const getSpecificGravity = (density) => {
    const cleaned = density.toString().replace(/,/, '.');
    const densityNumber = parseFloat(cleaned)

    switch (unit) {
      case constants.SPECIFIC_GRAVITY:
        return densityNumber;
      case constants.PLATO:
        return convertPlatoToSg(densityNumber);
      default:
        return undefined
    }
  }

  const getFermentationActivity = (mSgPerHour) => {
    switch (unit) {
      case constants.SPECIFIC_GRAVITY:
        return parseFloat(mSgPerHour);
      case constants.PLATO:
        return convertMilliSgPerHourToPlatoPerHour(mSgPerHour);
      default:
        return undefined
    }
  }

  const getFermentationActivityUnit = () => {
    switch (unit) {
      case constants.SPECIFIC_GRAVITY:
        return 'mSG/h';
      case constants.PLATO:
        return '°P/h';
      default:
        return undefined
    }
  }

  const getFermentationActivityString = (mSgPerHour, withUnit = true) => {
    const fermentationActivity = getFermentationActivity(mSgPerHour);
    if (!Number.isFinite(fermentationActivity)) return '--';

    let result;
    switch (unit) {
      case constants.SPECIFIC_GRAVITY:
        result = fermentationActivity.toFixed(2);
        break;
      case constants.PLATO:
        result = fermentationActivity.toFixed(2);
        break;
      default:
        return '--'
    }

    if (withUnit) {
      result += ` ${getFermentationActivityUnit()}`;
    }

    return result;
  }

  function convertMilliSgPerHourToPlatoPerHour(mSgPerHour) {
    const sgPerHour = mSgPerHour / 1000;
    const sgEquivalent = 1 + sgPerHour;
    return convertSgToPlato(sgEquivalent)
  }

  return {
    setDensityUnit,
    getSpecificGravity,
    getDensityMin,
    getDensityMax,
    getDensity,
    getDensityString,
    getDensityUnit,
    getFermentationActivity,
    getFermentationActivityString,
    getFermentationActivityUnit
  };
}

export default useDensityUtils;